@@ -1,58 +1,25 @@
 #include <config.h>
 
-/* PostgreSQL backend, by Cindy Marasco <cindy@getaclue.org> */
+/* PostgreSQL backend, by Cindy Marasco <cindy at getaclue.org> */
 
 #ifdef WITH_PGSQL
 
-#include "ftpd.h"
-#include "parser.h"
-#include "log_pgsql_p.h"
-#include "log_pgsql.h"
-#include "messages.h"
-#include "crypto.h"
+# include "ftpd.h"
+# include "parser.h"
+# include "log_pgsql_p.h"
+# include "log_pgsql.h"
+# include "messages.h"
+# include "crypto.h"
+# include "alt_arc4random.h"
+# include "utils.h"
 
-#ifdef WITH_DMALLOC
-# include <dmalloc.h>
-#endif
+# ifdef HAVE_LIBSODIUM
+#  include <sodium.h>
+# endif
 
-static size_t prv_PQescapeString(char * const to, const char * const from,
-                                 const size_t length)
-{
-    register const char *source = from;
-    register char *target = to;
-    size_t remaining = length;
-    
-    while (remaining > (size_t) 0U) {
-        switch (*source) {
-        case 0:
-            remaining = (size_t) 1U;
-            break;
-        case '\r':
-            *target++ = '\\';
-            *target++ = 'r';            
-            break;
-        case '\n':
-            *target++ = '\\';
-            *target++ = 'n';
-            break;
-        case '\b':
-            *target++ = '\\';
-            *target++ = 'b';
-            break;            
-        case '\\':
-        case '\'':
-        case '"':
-            *target++ = '\\';
-        default:
-            *target++ = *source;
-        }
-        source++;
-        remaining--;        
-    }    
-    *target = 0;
-    
-    return (size_t) (target - to);
-}
+# ifdef WITH_DMALLOC
+#  include <dmalloc.h>
+# endif
 
 static int pw_pgsql_validate_name(const char *name)
 {
@@ -65,40 +32,62 @@ static int pw_pgsql_validate_name(const char *name)
             (*name >= '0' && *name <= '9') ||
             *name == ' ' || *name == '-' ||
             *name == '_' || *name == '\'' || *name == '.' ||
-            *name == ':' || *name == '@') {
-            /* God bless the Perl 'unless' keyword */
+            *name == ':' || *name == '@' || *name == '+') {
+            /* God bless the Ruby 'unless' keyword */
         } else {
             return -1;
-        }            
+        }
         name++;
     } while (*name != 0);
-    
+
     return 0;
 }
 
-static char *pw_pgsql_escape_string(const char *from)
+static char *pw_pgsql_escape_string(PGconn * const id_sql_server,
+                                    const char *from)
 {
     size_t from_len;
     size_t to_len;
     char *to;
-    size_t tolen;    
-            
+    size_t tolen;
+    unsigned int t;
+    unsigned char t1, t2;
+    int error;
+
     if (from == NULL) {
         return NULL;
     }
     from_len = strlen(from);
     to_len = from_len * 2U + (size_t) 1U;
-    if ((to = malloc(to_len)) == NULL) {
+    if ((to = malloc(to_len + (size_t) 2U)) == NULL) {
         return NULL;
     }
-    tolen = prv_PQescapeString(to, from, from_len);
-    if (tolen >= to_len) {
+    t = zrand();
+    t1 = t & 0xff;
+    t2 = (t >> 8) & 0xff;
+    to[to_len] = (char) t1;
+    to[to_len + 1] = (char) t2;
+    /*
+     * I really hate giving a buffer without any size to a 3rd party function.
+     * The "to" buffer is allocated on the heap, not on the stack, if
+     * PQescapeStringConn() is buggy, the stack shouldn't be already
+     * smashed at this point, but data from other malloc can be corrupted and
+     * bad things can happen. It make sense to wipe this area as soon as
+     * possible instead of doing anything with the heap. We'll end up with
+     * a segmentation violation, but without any possible exploit.
+     */
+    tolen = PQescapeStringConn(id_sql_server, to, from, from_len, &error);
+    if (tolen >= to_len ||
+        (unsigned char) to[to_len] != t1 ||
+        (unsigned char) to[to_len + 1] != t2) {
         for (;;) {
             *to++ = 0;
         }
     }
     to[tolen] = 0;
-                
+    if (error != 0) {
+        return NULL;
+    }
     return to;
 }
 
@@ -146,40 +135,40 @@ static char *sqlsubst(const char *orig_str, char * const query,
                 }
                 if (ip_len <= (size_t) 0U) {
                     goto nextone;
-                }                
+                }
                 memcpy(query_pnt, ip, ip_len);
                 query_pnt += ip_len;
                 query_len -= ip_len;
                 goto nextone;
-            case 'p' :             
+            case 'p' :
                 if (port_len >= query_len) {
                     return NULL;
-                } 
+                }
                 if (port_len <= (size_t) 0U) {
                     goto nextone;
-                }               
+                }
                 memcpy(query_pnt, port, port_len);
                 query_pnt += port_len;
                 query_len -= port_len;
                 goto nextone;
-            case 'r' :             
+            case 'r' :
                 if (peer_ip_len >= query_len) {
                     return NULL;
-                } 
+                }
                 if (peer_ip_len <= (size_t) 0U) {
                     goto nextone;
-                }               
+                }
                 memcpy(query_pnt, peer_ip, peer_ip_len);
                 query_pnt += peer_ip_len;
                 query_len -= peer_ip_len;
                 goto nextone;
-            case 'd' :             
+            case 'd' :
                 if (decimal_ip_len >= query_len) {
                     return NULL;
-                } 
+                }
                 if (decimal_ip_len <= (size_t) 0U) {
                     goto nextone;
-                }               
+                }
                 memcpy(query_pnt, decimal_ip, decimal_ip_len);
                 query_pnt += decimal_ip_len;
                 query_len -= decimal_ip_len;
@@ -206,6 +195,75 @@ static char *sqlsubst(const char *orig_str, char * const query,
     return query;
 }
 
+static size_t pw_pgsql_escape_conninfo_(char * const to,
+                                        const char * const from,
+                                        const size_t length)
+{
+    const char *source = from;
+    char *target = to;
+    size_t remaining = length;
+
+    while (remaining > (size_t) 0U) {
+        switch (*source) {
+        case 0:
+            remaining = (size_t) 1U;
+            break;
+        case '\r':
+            *target++ = '\\';
+            *target++ = 'r';
+            break;
+        case '\n':
+            *target++ = '\\';
+            *target++ = 'n';
+            break;
+        case '\b':
+            *target++ = '\\';
+            *target++ = 'b';
+            break;
+        case '\'':
+            *target++ = '\'';
+            *target++ = '\'';
+            break;
+        case '\\':
+        case '"':
+            *target++ = '\\';
+        default:
+            *target++ = *source;
+        }
+        source++;
+        remaining--;
+    }
+    *target = 0;
+
+    return (size_t) (target - to);
+}
+
+static char *pw_pgsql_escape_conninfo(const char *from)
+{
+    size_t from_len;
+    size_t to_len;
+    char *to;
+    size_t tolen;
+
+    if (from == NULL) {
+        return NULL;
+    }
+    from_len = strlen(from);
+    to_len = from_len * 2U + (size_t) 1U;
+    if ((to = malloc(to_len)) == NULL) {
+        return NULL;
+    }
+    tolen = pw_pgsql_escape_conninfo_(to, from, from_len);
+    if (tolen >= to_len) {
+        for (;;) {
+            *to++ = 0;
+        }
+    }
+    to[tolen] = 0;
+
+    return to;
+}
+
 static int pw_pgsql_connect(PGconn ** const id_sql_server)
 {
     char *conninfo = NULL;
@@ -217,56 +275,45 @@ static int pw_pgsql_connect(PGconn ** const id_sql_server)
     int ret = -1;
 
     *id_sql_server = NULL;
-    
-    if ((escaped_server = pw_pgsql_escape_string(server)) == NULL ||
-        (escaped_db = pw_pgsql_escape_string(db)) == NULL ||        
-        (escaped_user = pw_pgsql_escape_string(user)) == NULL ||
-        (escaped_pw = pw_pgsql_escape_string(pw)) == NULL) {
+
+    if ((escaped_server = pw_pgsql_escape_conninfo(server)) == NULL ||
+        (escaped_db = pw_pgsql_escape_conninfo(db)) == NULL ||
+        (escaped_user = pw_pgsql_escape_conninfo(user)) == NULL ||
+        (escaped_pw = pw_pgsql_escape_conninfo(pw)) == NULL) {
         goto bye;
     }
-    
+
 #define PGSQL_CONNECT_FMTSTRING \
 "host='%s' port='%d' dbname='%s' user='%s' password='%s'"
-        
+
     sizeof_conninfo = sizeof PGSQL_CONNECT_FMTSTRING +
-        strlen(escaped_server) + (size_t) 5U + strlen(escaped_db) + 
+        strlen(escaped_server) + (size_t) 5U + strlen(escaped_db) +
         strlen(escaped_user) + strlen(escaped_pw);
     if ((conninfo = malloc(sizeof_conninfo)) == NULL) {
         goto bye;
     }
     if (SNCHECK(snprintf(conninfo, sizeof_conninfo,
-                         PGSQL_CONNECT_FMTSTRING, 
+                         PGSQL_CONNECT_FMTSTRING,
                          server, port, db, user, pw), sizeof_conninfo)) {
         goto bye;
-    }    
+    }
     if ((*id_sql_server = PQconnectdb(conninfo)) == NULL ||
         PQstatus(*id_sql_server) == CONNECTION_BAD) {
-        free(conninfo);
-	if (server_down == 0) {
-	    server_down++;
-	    logfile(LOG_ERR, MSG_SQL_DOWN);
-	}
+        if (server_down == 0) {
+            server_down++;
+            logfile(LOG_ERR, MSG_SQL_DOWN);
+        }
         goto bye;
     }
     server_down = 0;
     ret = 0;
-    
+
     bye:
-    if (conninfo != NULL) {
-        free(conninfo);
-    }
-    if (escaped_server != NULL) {
-        free(escaped_server);
-    }
-    if (escaped_db != NULL) {
-        free(escaped_db);
-    }
-    if (escaped_user != NULL) {
-        free(escaped_user);
-    }
-    if (escaped_pw != NULL) {
-        free(escaped_pw);
-    }
+    free(conninfo);
+    free(escaped_server);
+    free(escaped_db);
+    free(escaped_user);
+    free(escaped_pw);
 
     return ret;
 }
@@ -280,11 +327,11 @@ static int pw_pgsql_simplequery(PGconn * const id_sql_server,
         return -1;
     }
     if (PQresultStatus(result) != PGRES_COMMAND_OK) {
-        PQclear(result);        
-	return -1;
+        PQclear(result);
+    return -1;
     }
     PQclear(result);
-    
+
     return 0;
 }
 
@@ -309,7 +356,7 @@ static char *pw_pgsql_getquery(PGconn * const id_sql_server,
         goto bye;
     }
     if ((qresult = PQexec(id_sql_server, query)) == NULL) {
-        logfile(LOG_WARNING, MSG_SQL_WRONG_PARMS " : [%s]", query);        
+        logfile(LOG_WARNING, MSG_SQL_WRONG_PARMS " : [%s]", query);
         goto bye;
     }
     if (PQresultStatus(qresult) != PGRES_TUPLES_OK ||
@@ -324,13 +371,13 @@ static char *pw_pgsql_getquery(PGconn * const id_sql_server,
     }
     strncpy(answer, PQgetvalue(qresult, 0, 0), length - (size_t) 1U);
     answer[length - (size_t) 1U] = 0;
-    
+
     bye:
     if (qresult != NULL) {
         PQclear(qresult);
     }
-    
-    return answer;    
+
+    return answer;
 }
 
 void pw_pgsql_check(AuthResult * const result,
@@ -344,13 +391,13 @@ void pw_pgsql_check(AuthResult * const result,
     const char *gid = sql_default_gid; /* stored system group/gid */
     const char *dir = NULL;            /* stored home directory */
 #ifdef QUOTAS
-    const char *sqta_fs = NULL;        /* stored quota files */    
+    const char *sqta_fs = NULL;        /* stored quota files */
     const char *sqta_sz = NULL;        /* stored quota size */
-#endif    
+#endif
 #ifdef RATIOS
     const char *ratio_ul = NULL;       /* stored ratio UL */
     const char *ratio_dl = NULL;       /* stored ratio DL */
-#endif    
+#endif
 #ifdef THROTTLING
     const char *bandwidth_ul = NULL;   /* stored bandwidth UL */
     const char *bandwidth_dl = NULL;   /* stored bandwidth DL */
@@ -359,16 +406,17 @@ void pw_pgsql_check(AuthResult * const result,
     char *escaped_ip = NULL;
     char *escaped_port = NULL;
     char *escaped_peer_ip = NULL;
-    char *escaped_decimal_ip = NULL;    
+    char *escaped_decimal_ip = NULL;
     char *scrambled_password = NULL;
     int committed = 1;
-    int crypto_crypt = 0, crypto_plain = 0, crypto_md5 = 0;
+    int crypto_scrypt = 0, crypto_crypt = 0, crypto_md5 = 0, crypto_sha1 = 0,
+        crypto_plain = 0;
     unsigned long decimal_ip_num = 0UL;
     char decimal_ip[42];
     char hbuf[NI_MAXHOST];
     char pbuf[NI_MAXSERV];
     char phbuf[NI_MAXHOST];
-    
+
     result->auth_ok = 0;
     if (pw_pgsql_validate_name(account) != 0) {
         goto bye;
@@ -385,9 +433,9 @@ void pw_pgsql_check(AuthResult * const result,
     if (STORAGE_FAMILY(*peer) == AF_INET) {
         const unsigned char *decimal_ip_raw =
             (const unsigned char *) &(STORAGE_SIN_ADDR(*peer));
-        decimal_ip_num = (decimal_ip_raw[0] << 24) | 
-            (decimal_ip_raw[1] << 16) | (decimal_ip_raw[2] << 8) |
-            decimal_ip_raw[3];
+        decimal_ip_num = ((unsigned long) decimal_ip_raw[0] << 24) |
+            ((unsigned long) decimal_ip_raw[1] << 16) |
+            (decimal_ip_raw[2] << 8) | decimal_ip_raw[3];
         if (SNCHECK(snprintf(decimal_ip, sizeof decimal_ip,
                              "%lu", decimal_ip_num), sizeof decimal_ip)) {
             goto bye;
@@ -396,24 +444,24 @@ void pw_pgsql_check(AuthResult * const result,
     if (pw_pgsql_connect(&id_sql_server) != 0) {
         goto bye;
     }
-    if ((escaped_account = 
-         pw_pgsql_escape_string(account)) == NULL) {
+    if ((escaped_account =
+         pw_pgsql_escape_string(id_sql_server, account)) == NULL) {
         goto bye;
     }
-    if ((escaped_ip = 
-         pw_pgsql_escape_string(hbuf)) == NULL) {
+    if ((escaped_ip =
+         pw_pgsql_escape_string(id_sql_server, hbuf)) == NULL) {
         goto bye;
     }
-    if ((escaped_port = 
-         pw_pgsql_escape_string(pbuf)) == NULL) {
+    if ((escaped_port =
+         pw_pgsql_escape_string(id_sql_server, pbuf)) == NULL) {
         goto bye;
     }
-    if ((escaped_peer_ip = 
-         pw_pgsql_escape_string(phbuf)) == NULL) {
+    if ((escaped_peer_ip =
+         pw_pgsql_escape_string(id_sql_server, phbuf)) == NULL) {
         goto bye;
     }
-    if ((escaped_decimal_ip = 
-         pw_pgsql_escape_string(decimal_ip)) == NULL) {
+    if ((escaped_decimal_ip =
+         pw_pgsql_escape_string(id_sql_server, decimal_ip)) == NULL) {
         goto bye;
     }
     if (pw_pgsql_simplequery(id_sql_server, PGSQL_TRANSACTION_START) == 0) {
@@ -427,7 +475,7 @@ void pw_pgsql_check(AuthResult * const result,
     }
     if (uid == NULL) {
         uid = pw_pgsql_getquery(id_sql_server, sqlreq_getuid,
-                                escaped_account, escaped_ip, 
+                                escaped_account, escaped_ip,
                                 escaped_port, escaped_peer_ip,
                                 escaped_decimal_ip);
     }
@@ -451,39 +499,61 @@ void pw_pgsql_check(AuthResult * const result,
     }
     result->auth_ok--;                  /* -1 */
     if (strcasecmp(crypto, PASSWD_SQL_ANY) == 0) {
+        crypto_scrypt++;
         crypto_crypt++;
         crypto_md5++;
+        crypto_sha1++;
+    } else if (strcasecmp(crypto, PASSWD_SQL_SCRYPT) == 0) {
+        crypto_scrypt++;
     } else if (strcasecmp(crypto, PASSWD_SQL_CRYPT) == 0) {
         crypto_crypt++;
     } else if (strcasecmp(crypto, PASSWD_SQL_MD5) == 0) {
         crypto_md5++;
+    } else if (strcasecmp(crypto, PASSWD_SQL_SHA1) == 0) {
+        crypto_sha1++;
     } else {                           /* default to plaintext */
         crypto_plain++;
     }
+#ifdef HAVE_LIBSODIUM
+    if (crypto_scrypt != 0) {
+        if (crypto_pwhash_scryptsalsa208sha256_str_verify
+            (spwd, password, strlen(password)) == 0) {
+            goto auth_ok;
+        }
+    }
+#endif
     if (crypto_crypt != 0) {
-        register const char *crypted;
-        
+        const char *crypted;
+
         if ((crypted = (const char *) crypt(password, spwd)) != NULL &&
-            strcmp(crypted, spwd) == 0) {
+            pure_strcmp(crypted, spwd) == 0) {
             goto auth_ok;
         }
     }
     if (crypto_md5 != 0) {
-        register const char *crypted;
-        
+        const char *crypted;
+
         if ((crypted = (const char *) crypto_hash_md5(password, 1)) != NULL &&
-            strcmp(crypted, spwd) == 0) {
+            pure_strcmp(crypted, spwd) == 0) {
+            goto auth_ok;
+        }
+    }
+    if (crypto_sha1 != 0) {
+        const char *crypted;
+
+        if ((crypted = (const char *) crypto_hash_sha1(password, 1)) != NULL &&
+            pure_strcmp(crypted, spwd) == 0) {
             goto auth_ok;
         }
     }
     if (crypto_plain != 0) {
         if (*password != 0 &&    /* refuse null cleartext passwords */
-            strcmp(password, spwd) == 0) {
+            pure_strcmp(password, spwd) == 0) {
             goto auth_ok;
         }
     }
     goto bye;
-    
+
     auth_ok:
     /*
      * do *NOT* accept root uid/gid - if the database is compromized, the FTP
@@ -492,7 +562,7 @@ void pw_pgsql_check(AuthResult * const result,
     result->uid = (uid_t) strtoul(uid, NULL, 10);
     if (result->uid <= (uid_t) 0) {
         struct passwd *pw;
-        
+
         if ((pw = getpwnam(uid)) == NULL || pw->pw_uid <= (uid_t) 0) {
             goto bye;
         }
@@ -501,21 +571,21 @@ void pw_pgsql_check(AuthResult * const result,
     result->gid = (gid_t) strtoul(gid, NULL, 10);
     if (result->gid <= (gid_t) 0) {
         struct group *gr;
-        
+
         if ((gr = getgrnam(gid)) == NULL || gr->gr_gid <= (gid_t) 0) {
             goto bye;
         }
         result->gid = gr->gr_gid;
-    }    
+    }
     result->dir = dir;
-    dir = NULL;    
+    dir = NULL;
 #ifdef QUOTAS
     if ((sqta_fs = pw_pgsql_getquery(id_sql_server, sqlreq_getqta_fs,
                                      escaped_account, escaped_ip,
                                      escaped_port, escaped_peer_ip,
                                      escaped_decimal_ip)) != NULL) {
         const unsigned long long q = strtoull(sqta_fs, NULL, 10);
-        
+
         if (q > 0ULL) {
             result->user_quota_files = q;
             result->quota_files_changed = 1;
@@ -526,20 +596,20 @@ void pw_pgsql_check(AuthResult * const result,
                                      escaped_port, escaped_peer_ip,
                                      escaped_decimal_ip)) != NULL) {
         const unsigned long long q = strtoull(sqta_sz, NULL, 10);
-        
+
         if (q > 0ULL) {
             result->user_quota_size = q * (1024UL * 1024UL);
             result->quota_size_changed = 1;
         }
     }
-#endif           
+#endif
 #ifdef RATIOS
     if ((ratio_ul = pw_pgsql_getquery(id_sql_server, sqlreq_getratio_ul,
                                       escaped_account, escaped_ip,
                                       escaped_port, escaped_peer_ip,
                                       escaped_decimal_ip)) != NULL) {
         const unsigned int q = (unsigned int) strtoul(ratio_ul, NULL, 10);
-        
+
         if (q > 0U) {
             result->ratio_upload = q;
             result->ratio_ul_changed = 1;
@@ -550,7 +620,7 @@ void pw_pgsql_check(AuthResult * const result,
                                       escaped_port, escaped_peer_ip,
                                       escaped_decimal_ip)) != NULL) {
         const unsigned int q = (unsigned int) strtoul(ratio_dl, NULL, 10);
-        
+
         if (q > 0U) {
             result->ratio_download = q;
             result->ratio_dl_changed = 1;
@@ -563,7 +633,7 @@ void pw_pgsql_check(AuthResult * const result,
                                           escaped_port, escaped_peer_ip,
                                           escaped_decimal_ip)) != NULL) {
         const unsigned long q = (unsigned long) strtoul(bandwidth_ul, NULL, 10);
-        
+
         if (q > 0UL) {
             result->throttling_bandwidth_ul = q * 1024UL;
             result->throttling_ul_changed = 1;
@@ -574,15 +644,15 @@ void pw_pgsql_check(AuthResult * const result,
                                           escaped_port, escaped_peer_ip,
                                           escaped_decimal_ip)) != NULL) {
         const unsigned long q = (unsigned long) strtoul(bandwidth_dl, NULL, 10);
-        
+
         if (q > 0UL) {
             result->throttling_bandwidth_dl = q * 1024UL;
             result->throttling_dl_changed = 1;
         }
     }
-#endif    
+#endif
     result->slow_tilde_expansion = 1;
-    result->auth_ok =- result->auth_ok;
+    result->auth_ok = -result->auth_ok;
     bye:
     if (committed == 0) {
         (void) pw_pgsql_simplequery(id_sql_server, PGSQL_TRANSACTION_END);
@@ -590,69 +660,41 @@ void pw_pgsql_check(AuthResult * const result,
     if (id_sql_server != NULL) {
         PQfinish(id_sql_server);
     }
-    if (spwd != NULL) {
-        free((void *) spwd);
-    }
-    if (uid != NULL) {
+    free((void *) spwd);
+    if (uid != sql_default_uid) {
         free((void *) uid);
     }
-    if (gid != NULL) {
+    if (gid != sql_default_gid) {
         free((void *) gid);
     }
-    if (dir != NULL) {
-        free((void *) dir);
-    }
-    if (scrambled_password != NULL) {
-        free(scrambled_password);
-    }
+    free((void *) dir);
+    free(scrambled_password);
 #ifdef QUOTAS
-    if (sqta_fs != NULL) {
-        free((void *) sqta_fs);
-    }
-    if (sqta_sz != NULL) {
-        free((void *) sqta_sz);
-    }
-#endif    
+    free((void *) sqta_fs);
+    free((void *) sqta_sz);
+#endif
 #ifdef RATIOS
-    if (ratio_ul != NULL) {
-        free((void *) ratio_ul);
-    }
-    if (ratio_dl != NULL) {
-        free((void *) ratio_dl);
-    }
-#endif    
+    free((void *) ratio_ul);
+    free((void *) ratio_dl);
+#endif
 #ifdef THROTTLING
-    if (bandwidth_ul != NULL) {
-        free((void *) bandwidth_ul);
-    }
-    if (bandwidth_dl != NULL) {
-        free((void *) bandwidth_dl);
-    }
-#endif    
-    if (escaped_account != NULL) {
-        free((void *) escaped_account);
-    }
-    if (escaped_ip != NULL) {
-        free((void *) escaped_ip);
-    }
-    if (escaped_port != NULL) {
-        free((void *) escaped_port);
-    }
-    if (escaped_peer_ip != NULL) {
-        free((void *) escaped_peer_ip);
-    }
-    if (escaped_decimal_ip != NULL) {
-        free((void *) escaped_decimal_ip);
-    }    
+    free((void *) bandwidth_ul);
+    free((void *) bandwidth_dl);
+#endif
+    free((void *) escaped_account);
+    free((void *) escaped_ip);
+    free((void *) escaped_port);
+    free((void *) escaped_peer_ip);
+    free((void *) escaped_decimal_ip);
 }
 
 void pw_pgsql_parse(const char * const file)
 {
     if (generic_parser(file, pgsql_config_keywords) != 0) {
         die(421, LOG_ERR, MSG_CONF_ERR ": " MSG_ILLEGAL_CONFIG_FILE_SQL ": %s" , file);
-    }    
+    }
     if (server == NULL ) {
-        die(421, LOG_ERR, MSG_SQL_MISSING_SERVER);        
+        die(421, LOG_ERR, MSG_SQL_MISSING_SERVER);
     }
     if (port_s != NULL) {
         port = atoi(port_s);
@@ -664,13 +706,13 @@ void pw_pgsql_parse(const char * const file)
     }
 }
 
-#define ZFREE(X) if ((X) != NULL) { free(X); (X) = NULL; }
+#define ZFREE(X) do { free(X); (X) = NULL; } while (0)
 
 void pw_pgsql_exit(void)
 {
     ZFREE(server);
     ZFREE(port_s);
-    port = -1;    
+    port = -1;
     ZFREE(user);
     ZFREE(pw);
     ZFREE(db);
@@ -687,11 +729,11 @@ void pw_pgsql_exit(void)
 #endif
 #ifdef RATIOS
     ZFREE(sqlreq_getratio_ul);
-    ZFREE(sqlreq_getratio_dl);    
+    ZFREE(sqlreq_getratio_dl);
 #endif
 #ifdef THROTTLING
     ZFREE(sqlreq_getbandwidth_ul);
-    ZFREE(sqlreq_getbandwidth_dl);    
+    ZFREE(sqlreq_getbandwidth_dl);
 #endif
 }
 #else
